#ifndef SCALAR_H
#define SCALAR_H

#ifdef _MSC_VER
	#pragma once
#endif

class IVDShaderLogOutput;

enum {
	kVDShaderInterpOp_end,
	kVDShaderInterpOp_mov,
	kVDShaderInterpOp_add,
	kVDShaderInterpOp_sub,
	kVDShaderInterpOp_mul,
	kVDShaderInterpOp_rsq,
	kVDShaderInterpOp_rcp,
	kVDShaderInterpOp_frc,
	kVDShaderInterpOp_neg,
	kVDShaderInterpOp_min,
	kVDShaderInterpOp_max,
	kVDShaderInterpOp_abs,
	kVDShaderInterpOp_log,
	kVDShaderInterpOp_exp,
	kVDShaderInterpOp_cmp,
	kVDShaderInterpOp_texld,
	kVDShaderInterpOp_unpackb,
	kVDShaderInterpOp_unpackg,
	kVDShaderInterpOp_unpackr,
	kVDShaderInterpOp_unpacka,
	kVDShaderInterpOp_beginloop
};

struct VDShaderInterpreterTransformedInsn {
	int op;
	int dst;
	int src1;
	int src2;
	int src3;
};

struct VDShaderScalarForm {
	VDShaderScalarForm();
	~VDShaderScalarForm();

	void Clear();

	typedef VDShaderInterpreterTransformedInsn TransformedInsn;
	typedef vdfastvector<TransformedInsn> Insns;
	Insns mInsns;

	vdfastvector<float> mConstants;
	int mTempCount;

	int mInputCount;		// not valid until end
};

class VDShaderScalarTransformer {
public:
	VDShaderScalarTransformer(VDShaderScalarForm& dst);
	~VDShaderScalarTransformer();

	bool LoadShader(const VDShaderFunction& func, const VDShaderBindingElement *pElements, int numElements, IVDShaderLogOutput *out);

protected:
	void mov(int dst, int src);
	void add(int dst, int src1, int src2);
	void sub(int dst, int src1, int src2);
	void mul(int dst, int src1, int src2);
	void min(int dst, int src1, int src2);
	void max(int dst, int src1, int src2);
	void rsq(int dst, int src);
	void rcp(int dst, int src);
	void frc(int dst, int src);
	void log(int dst, int src);
	void exp(int dst, int src);
	void neg(int dst, int src);
	void abs(int dst, int src);
	void cmp(int dst, int src1, int src2, int src3);
	void texld(int dst, int srcu, int srcv);
	void unpackr(int dst, int src);
	void unpackg(int dst, int src);
	void unpackb(int dst, int src);
	void unpacka(int dst, int src);
	void beginloop();

	void push_insn(int op, int dst, int src1, int src2, int src3 = 0);

	VDShaderScalarForm&	mShader;
};

#endif
