//	VDShader - Custom shader video filter for VirtualDub
//	Copyright (C) 2007 Avery Lee
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "vecarith.h"

VDShaderConstant operator-(const VDShaderConstant& x) {
	VDShaderConstant r;
	r.v[0] = -x.v[0];
	r.v[1] = -x.v[1];
	r.v[2] = -x.v[2];
	r.v[3] = -x.v[3];
	return r;
}

VDShaderConstant operator+(const VDShaderConstant& x, const VDShaderConstant& y) {
	VDShaderConstant r;
	r.v[0] = x.v[0] + y.v[0];
	r.v[1] = x.v[1] + y.v[1];
	r.v[2] = x.v[2] + y.v[2];
	r.v[3] = x.v[3] + y.v[3];
	return r;
}

VDShaderConstant operator-(const VDShaderConstant& x, const VDShaderConstant& y) {
	VDShaderConstant r;
	r.v[0] = x.v[0] - y.v[0];
	r.v[1] = x.v[1] - y.v[1];
	r.v[2] = x.v[2] - y.v[2];
	r.v[3] = x.v[3] - y.v[3];
	return r;
}

VDShaderConstant operator*(const VDShaderConstant& x, const VDShaderConstant& y) {
	VDShaderConstant r;
	r.v[0] = x.v[0] * y.v[0];
	r.v[1] = x.v[1] * y.v[1];
	r.v[2] = x.v[2] * y.v[2];
	r.v[3] = x.v[3] * y.v[3];
	return r;
}

VDShaderConstant& operator+=(VDShaderConstant& x, const VDShaderConstant& y) {
	x.v[0] += y.v[0];
	x.v[1] += y.v[1];
	x.v[2] += y.v[2];
	x.v[3] += y.v[3];
	return x;
}

VDShaderConstant& operator-=(VDShaderConstant& x, const VDShaderConstant& y) {
	x.v[0] -= y.v[0];
	x.v[1] -= y.v[1];
	x.v[2] -= y.v[2];
	x.v[3] -= y.v[3];
	return x;
}

VDShaderConstant& operator*=(VDShaderConstant& x, const VDShaderConstant& y) {
	x.v[0] *= y.v[0];
	x.v[1] *= y.v[1];
	x.v[2] *= y.v[2];
	x.v[3] *= y.v[3];
	return x;
}

namespace nsVDShaderArith {
	VDShaderConstant div1(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[0] / y.v[0];
		r.v[1] = 0.0f;
		r.v[2] = 0.0f;
		r.v[3] = 0.0f;
		return r;
	}

	VDShaderConstant div2(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[0] / y.v[0];
		r.v[1] = x.v[1] / y.v[1];
		r.v[2] = 0.0f;
		r.v[3] = 0.0f;
		return r;
	}

	VDShaderConstant div3(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[0] / y.v[0];
		r.v[1] = x.v[1] / y.v[1];
		r.v[2] = x.v[2] / y.v[2];
		r.v[3] = 0.0f;
		return r;
	}

	VDShaderConstant div4(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[0] / y.v[0];
		r.v[1] = x.v[1] / y.v[1];
		r.v[2] = x.v[2] / y.v[2];
		r.v[3] = x.v[3] / y.v[3];
		return r;
	}

	VDShaderConstant dp4(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[0]*y.v[0] + x.v[1]*y.v[1] + x.v[2]*y.v[2] + x.v[3]*y.v[3];
		r.v[1] = 0.0f;
		r.v[2] = 0.0f;
		r.v[3] = 0.0f;
		return r;
	}

	VDShaderConstant dp3(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[0]*y.v[0] + x.v[1]*y.v[1] + x.v[2]*y.v[2];
		r.v[1] = 0.0f;
		r.v[2] = 0.0f;
		r.v[3] = 0.0f;
		return r;
	}

	VDShaderConstant dp2(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[0]*y.v[0] + x.v[1]*y.v[1];
		r.v[1] = 0.0f;
		r.v[2] = 0.0f;
		r.v[3] = 0.0f;
		return r;
	}

	VDShaderConstant swizzle(const VDShaderConstant& x, unsigned char swiz) {
		VDShaderConstant r;
		r.v[0] = x.v[(swiz     ) & 3];
		r.v[1] = x.v[(swiz >> 2) & 3];
		r.v[2] = x.v[(swiz >> 4) & 3];
		r.v[3] = x.v[(swiz >> 6) & 3];
		return r;
	}

	VDShaderConstant min(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[0] < y.v[0] ? x.v[0] : y.v[0];
		r.v[1] = x.v[1] < y.v[1] ? x.v[1] : y.v[1];
		r.v[2] = x.v[2] < y.v[2] ? x.v[2] : y.v[2];
		r.v[3] = x.v[3] < y.v[3] ? x.v[3] : y.v[3];
		return r;
	}

	VDShaderConstant max(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[0] > y.v[0] ? x.v[0] : y.v[0];
		r.v[1] = x.v[1] > y.v[1] ? x.v[1] : y.v[1];
		r.v[2] = x.v[2] > y.v[2] ? x.v[2] : y.v[2];
		r.v[3] = x.v[3] > y.v[3] ? x.v[3] : y.v[3];
		return r;
	}

	VDShaderConstant abs(const VDShaderConstant& x) {
		VDShaderConstant r;
		r.v[0] = fabsf(x.v[0]);
		r.v[1] = fabsf(x.v[1]);
		r.v[2] = fabsf(x.v[2]);
		r.v[3] = fabsf(x.v[3]);
		return r;
	}

	VDShaderConstant cross(const VDShaderConstant& x, const VDShaderConstant& y) {
		VDShaderConstant r;
		r.v[0] = x.v[1]*y.v[2] - x.v[2]*y.v[1];
		r.v[1] = x.v[2]*y.v[0] - x.v[0]*y.v[2];
		r.v[2] = x.v[0]*y.v[1] - x.v[1]*y.v[0];
		r.v[3] = 0;
		return r;
	}

	VDShaderConstant lerp(const VDShaderConstant& x, const VDShaderConstant& y, const VDShaderConstant& z) {
		VDShaderConstant r;
		r.v[0] = x.v[0] + (y.v[0] - x.v[0])*z.v[0];
		r.v[1] = x.v[1] + (y.v[1] - x.v[1])*z.v[1];
		r.v[2] = x.v[2] + (y.v[2] - x.v[2])*z.v[2];
		r.v[3] = x.v[3] + (y.v[3] - x.v[3])*z.v[3];
		return r;
	}
}